package HTML::FormHandler::Page::Simple;
# ABSTRACT: used by Wizard

use Moose;
extends 'HTML::FormHandler::Page';

1;

__END__
=pod

=head1 NAME

HTML::FormHandler::Page::Simple - used by Wizard

=head1 VERSION

version 0.36000

=head1 AUTHOR

FormHandler Contributors - see HTML::FormHandler

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2012 by Gerda Shank.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

